import 'dart:async';
import 'dart:convert';

import 'package:dio/dio.dart';
import 'package:flutter/foundation.dart';
import 'package:lib_common_fx/network/config_interface.dart';
import 'package:lib_common_fx/network/log_interceptor.dart' as log;
import 'package:lib_common_fx/network/response.dart';
import 'package:lib_common_fx/utils/log_util.dart';

///Author: wangys
///Date: 2022-10-12 09:05:44
///Description:
///
// 第一步初始化
// class NetConfig extends NetConfigInterface {
//   @override
//   int get successCode => 0;

//   @override
//   String getHost() => "http://host";

//   @override
//   int getResponseCode(Map<String, dynamic> map) {
//     return JsonConvert.fromJsonAsT<int>(json['code']) ?? -1;
//   }

//   @override
//   T? getResponseData<T>(Map<String, dynamic> map) {
//     return JsonConvert.fromJsonAsT<T>(json['data']);
//   }

//   @override
//   String getResponseMsg(Map<String, dynamic> map) {
//     return JsonConvert.fromJsonAsT<String>(json['msg']) ?? "";
//   }
//   //设置默认header
//   Map<String, String> defaultHeader() {
//     return {};
//   }
// }
//
// void main() {
//   Net.initConfig(NetConfig());
//   runApp(app);
// }
// 发送请求
// void doNet()async {
//   NetResponse response = await Net.post<UserInfo>("/user/login", data: {"phone": "12312341234", "pwd": "123456"});
//   var info = response.data!;
// }
//如果不关心返回值
//doNet() async{
//  await Net.get(url);
//}

class Net {
  late Dio _dio;
  late NetConfigInterface config;
  bool hasInit = false;
  static final Net _singletonNetClient = Net._internal();

  Net._internal();

  void _init(NetConfigInterface config) {
    this.config = config;
    var options = BaseOptions(
        baseUrl: config.getHost(),
        connectTimeout: config.connectTimeOut,
        receiveTimeout: config.readTimeOut,
        sendTimeout: config.writeTimeOut);
    _dio = Dio(options)..interceptors.add(log.LogInterceptor(config.logEnable));
  }

  static void initConfig(NetConfigInterface config) {
    _singletonNetClient._init(config);
  }

  Future<T> _request<T>(
    String url, {
    required String method,
    Map<String, dynamic>? queryParameters,
    dynamic data,
    bool convertDate = true,
    Map<String, dynamic>? headers,
  }) async {
    try {
      Options options = Options()
        ..method = method
        ..headers = config.defaultHeader();
      if (headers != null) {
        options.headers!.addAll(headers);
      }
      if (convertDate == true) {
        data = _convertRequestData(data);
      }

      Response response = await _dio.request(
        url,
        queryParameters: queryParameters,
        data: data,
        options: options,
      );
      if (response.statusCode == 200) {
        T result = _convertResponseData(response);
        return result;
      } else {
        throw NetError(
            (response.statusCode ?? 0) * -1, "HttpCode:${response.statusCode}");
      }
    } catch (e, s) {
      config.onError(e);
      if (kDebugMode) print(s);
      rethrow;
    }
  }

  ///将请求 data 数据先使用 jsonEncode 转换为字符串，再使用 jsonDecode 方法将字符串转换为 Map。
  _convertRequestData(data) {
    if (data != null) {
      data = jsonDecode(jsonEncode(data));
    }
    return data;
  }

  T _convertResponseData<T>(Response response) {
    if (T.toString() == (RawData).toString()) {
      //返回原始类型
      RawData raw = RawData();
      raw.value = response.data;
      return raw as T;
    } else if (response.data is Map) {
      //返回 T?
      Map<String, dynamic> map = response.data as Map<String, dynamic>;
      int code = config.getResponseCode(map);
      String msg = config.getResponseMsg(map);
      if (code == config.successCode) {
        final String type = T.toString();
        if (type == 'ListResponse' || type.startsWith("ListResponse<")) {
          return config.coverListResponse(map) as T;
        } else {
          return config.getResponseData<T>(map)!;
        }
      } else {
        throw NetError(code, msg);
      }
    } else {
      throw NetError(-1, "unsupported type ${T.toString()}");
    }
  }

  static Future<T> get<T>(
    String url, {
    Map<String, dynamic>? queryMap,
    Map<String, dynamic>? headers,
  }) {
    return _singletonNetClient._request(
      url,
      method: "GET",
      queryParameters: queryMap,
      headers: headers,
    );
  }

  static Future<T> post<T>(
    String url, {
    Map<String, dynamic>? queryMap,
    dynamic data,
    Map<String, dynamic>? headers,
  }) {
    return _singletonNetClient._request(
      url,
      method: "POST",
      queryParameters: queryMap,
      data: data,
      headers: headers,
    );
  }

  static Future<T> delete<T>(
    String url, {
    Map<String, dynamic>? queryMap,
    dynamic data,
    Map<String, dynamic>? headers,
  }) {
    return _singletonNetClient._request(
      url,
      method: "DELETE",
      queryParameters: queryMap,
      data: data,
      headers: headers,
    );
  }

  static Future<T> put<T>(
    String url, {
    Map<String, dynamic>? queryMap,
    dynamic data,
    Map<String, dynamic>? headers,
  }) {
    return _singletonNetClient._request(
      url,
      method: "PUT",
      queryParameters: queryMap,
      data: data,
      headers: headers,
    );
  }

  ///FormData data = FormData.fromMap({
  ///      'image': MultipartFile.fromFileSync(filePath),
  ///});
  ///Net.upload<Result>(url,data:form);
  static Future<T> upload<T>(
    String url, {
    required FormData data,
    Map<String, dynamic>? headers,
  }) {
    return _singletonNetClient._request(
      url,
      method: "POST",
      data: data,
      convertDate: false,
      headers: headers,
    );
  }

  static Future<ListResponse<T>> getList<T>(
    String url, {
    Map<String, dynamic>? queryMap,
    Map<String, dynamic>? headers,
  }) async {
    var result = await _singletonNetClient._request<ListResponse>(
      url,
      method: "GET",
      queryParameters: queryMap,
      headers: headers,
    );
    var list = _singletonNetClient.config.coverList<T>(result.list);
    return ListResponse.transform(result, list);
  }

  static Future<ListResponse<T>> postList<T>(
    String url, {
    Map<String, dynamic>? queryMap,
    dynamic data,
    Map<String, dynamic>? headers,
  }) async {
    var result = await _singletonNetClient._request(
      url,
      method: "POST",
      queryParameters: queryMap,
      data: data,
      headers: headers,
    );
    var list = _singletonNetClient.config.coverList<T>(result.list);
    return ListResponse.transform(result, list);
  }
}
