import 'package:flutter/material.dart';
import 'package:lib_common_fx/lib_common_fx.dart';
import 'package:lib_common_fx/utils/image_utils.dart';

///Author: wangys
///Date: 2022-10-17 09:30:41
///Description: FutureBuilder封装

///
///```dart
///  final key = GlobalKey<FxFutureBuilderState>();
///  key.currentState!.reload();
///
///  FxFutureBuilder(
///   key: key
///   fetchFun: _fetchData,
///   content: _content(),
///  )
///  Future _fetchData() async {
///    await Future.delayed(const Duration(seconds: 3));
///  }
///```dart
///

class FxFutureBuilder extends StatefulWidget {
  const FxFutureBuilder({
    Key? key,
    required this.fetchFun,
    required this.content,
    this.retry = true,
    this.error,
    this.loading,
  }) : super(key: key);

  final Future Function() fetchFun;
  final Builder content;
  final bool retry;
  final Widget? error;
  final Widget? loading;

  @override
  State<FxFutureBuilder> createState() => FxFutureBuilderState();
}

class FxFutureBuilderState extends State<FxFutureBuilder> {
  late Future _future;

  @override
  void initState() {
    super.initState();
    _future = widget.fetchFun();
  }

  @override
  Widget build(BuildContext context) {
    return FutureBuilder(
      future: _future,
      builder: (context, snapshot) {
        if (snapshot.connectionState == ConnectionState.done) {
          if (snapshot.hasError) {
            return widget.error == null ? _error() : widget.error!;
          } else {
            return widget.content;
          }
        }
        return widget.loading == null ? _loading() : widget.loading!;
      },
    );
  }

  void reload() {
    _future = widget.fetchFun();
    setState(() {});
  }

  Widget _error() {
    return GestureDetector(
      onTap: () {
        if (widget.retry) {
          _future = widget.fetchFun();
          setState(() {});
        }
      },
      child: Container(
        constraints: const BoxConstraints.expand(),
        decoration: const BoxDecoration(),
        alignment: const Alignment(0.0, -0.4),
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: [
            ImageUtil.assetImage("images/icon_net_error.png",
                width: 300, height: 200),
            const SizedBox(height: 12),
            widget.retry ? const Text("加载出错，点击界面重试") : const Text("加载出错"),
          ],
        ),
      ),
    );
  }

  Widget _loading() {
    return const Align(
        alignment: Alignment(0.0, 0.0),
        child: CircularProgressIndicator(color: FXColor.main_blue));
  }
}
